@@ -13,14 +13,13 @@ import json
 import os
 import sys
 import time
-from datetime import datetime
 
 import zmq
 from zmq.eventloop.zmqstream import ZMQStream
 
 # internal:
 from ipython_genutils.importstring import import_item
-from jupyter_client.jsonutil import extract_dates
+from ..util import extract_dates
 from jupyter_client.localinterfaces import localhost
 from ipython_genutils.py3compat import cast_bytes, unicode_type, iteritems, buffer_to_bytes_py2
 from traitlets import (
@@ -78,7 +77,7 @@ def init_record(msg):
         'content': msg['content'],
         'metadata': msg['metadata'],
         'buffers': msg['buffers'],
-        'submitted': header['date'],
+        'submitted': util.ensure_timezone(header['date']),
         'client_uuid' : None,
         'engine_uuid' : None,
         'started': None,
@@ -377,7 +376,6 @@ class Hub(SessionFactory):
     tasks=Dict() # pending msg_ids submitted as tasks, keyed by client_id
     completed=Dict() # completed msg_ids keyed by engine_id
     all_completed=Set() # completed msg_ids keyed by engine_id
-    dead_engines=Set() # completed msg_ids keyed by engine_id
     unassigned=Set() # set of task msg_ds not yet assigned a destination
     incoming_registrations=Dict()
     registration_timeout=Integer()
@@ -440,7 +438,7 @@ class Hub(SessionFactory):
                                 'registration_request' : self.register_engine,
                                 'unregistration_request' : self.unregister_engine,
                                 'connection_request': self.connection_request,
-                                'become_distributed_request': self.become_distributed,
+                                'become_dask_request': self.become_dask,
                                 'stop_distributed_request': self.stop_distributed,
         }
 
@@ -449,20 +447,28 @@ class Hub(SessionFactory):
 
         self.log.info("hub::created hub")
     
-    @property
-    def _next_id(self):
-        """gemerate a new ID.
+    def new_engine_id(self, requested_id=None):
+        """generate a new engine integer id.
 
-        No longer reuse old ids, just count from 0."""
+        No longer reuse old ids, just count from 0.
+
+        If an id is requested and available, use that.
+        Otherwise, use the counter.
+        """
+        if requested_id is not None:
+            if requested_id in self.engines:
+                self.log.warning("Engine id %s in use by engine with uuid=%s",
+                     requested_id, self.engines[requested_id].uuid)
+            elif requested_id in {
+                ec.id for ec in self.incoming_registrations.values()
+            }:
+                self.log.warning("Engine id %s registration already pending", requested_id)
+            else:
+                self._idcounter = max(requested_id + 1, self._idcounter)
+                return requested_id
         newid = self._idcounter
         self._idcounter += 1
         return newid
-        # newid = 0
-        # incoming = [id[0] for id in itervalues(self.incoming_registrations)]
-        # # print newid, self.ids, self.incoming_registrations
-        # while newid in self.ids or newid in incoming:
-        #     newid += 1
-        # return newid
     
     #-----------------------------------------------------------------------------
     # message validation
@@ -581,7 +587,7 @@ class Hub(SessionFactory):
         self.log.debug("heartbeat::handle_heart_failure(%r)", heart)
         eid = self.hearts.get(heart, None)
         uuid = self.engines[eid].uuid
-        if eid is None or self.keytable[eid] in self.dead_engines:
+        if eid is None:
             self.log.info("heartbeat::ignoring heart failure %r (not an engine or already dead)", heart)
         else:
             self.unregister_engine(heart, dict(content=dict(id=eid, queue=uuid)))
@@ -671,13 +677,13 @@ class Hub(SessionFactory):
         # update record anyway, because the unregistration could have been premature
         rheader = msg['header']
         md = msg['metadata']
-        completed = rheader['date']
+        completed = util.ensure_timezone(rheader['date'])
         started = extract_dates(md.get('started', None))
         result = {
             'result_header' : rheader,
             'result_metadata': md,
             'result_content': msg['content'],
-            'received': datetime.now(),
+            'received': util.utcnow(),
             'started' : started,
             'completed' : completed
         }
@@ -777,7 +783,7 @@ class Hub(SessionFactory):
                     self.completed[eid].append(msg_id)
                 if msg_id in self.tasks[eid]:
                     self.tasks[eid].remove(msg_id)
-            completed = header['date']
+            completed = util.ensure_timezone(header['date'])
             started = extract_dates(md.get('started', None))
             result = {
                 'result_header' : header,
@@ -785,7 +791,7 @@ class Hub(SessionFactory):
                 'result_content': msg['content'],
                 'started' : started,
                 'completed' : completed,
-                'received' : datetime.now(),
+                'received' : util.utcnow(),
                 'engine_uuid': engine_uuid,
             }
 
@@ -817,20 +823,12 @@ class Hub(SessionFactory):
         #     self.log.debug("task::task %r not listed as MIA?!"%(msg_id))
 
         self.tasks[eid].append(msg_id)
-        # self.pending[msg_id][1].update(received=datetime.now(),engine=(eid,engine_uuid))
         try:
             self.db.update_record(msg_id, dict(engine_uuid=engine_uuid))
         except Exception:
             self.log.error("DB Error saving task destination %r", msg_id, exc_info=True)
 
 
-    def mia_task_request(self, idents, msg):
-        raise NotImplementedError
-        client_id = idents[0]
-        # content = dict(mia=self.mia,status='ok')
-        # self.session.send('mia_reply', content=content, idents=client_id)
-
-
     #--------------------- IOPub Traffic ------------------------------
 
     def monitor_iopub_message(self, topics, msg):
@@ -915,7 +913,6 @@ class Hub(SessionFactory):
         content = dict(status='ok')
         jsonable = {}
         for k,v in iteritems(self.keytable):
-            if v not in self.dead_engines:
                 jsonable[str(k)] = v
         content['engines'] = jsonable
         self.session.send(self.query, 'connection_reply', content, parent=msg, ident=client_id)
@@ -929,7 +926,7 @@ class Hub(SessionFactory):
             self.log.error("registration::queue not specified", exc_info=True)
             return
 
-        eid = self._next_id
+        eid = self.new_engine_id(content.get('id'))
 
         self.log.debug("registration::register_engine(%i, %r)", eid, uuid)
 
@@ -992,7 +989,6 @@ class Hub(SessionFactory):
         
         uuid = self.keytable[eid]
         content=dict(id=eid, uuid=uuid)
-        self.dead_engines.add(uuid)
         
         #stop the heartbeats
         self.hearts.pop(uuid, None)
@@ -1003,8 +999,12 @@ class Hub(SessionFactory):
             self.loop.time() + self.registration_timeout,
             lambda : self._handle_stranded_msgs(eid, uuid),
         )
-        ############## TODO: HANDLE IT ################
-        
+
+        # cleanup mappings
+        self.by_ident.pop(uuid, None)
+        self.engines.pop(eid, None)
+        self.keytable.pop(eid, None)
+
         self._save_engine_state()
 
         if self.notifier:
@@ -1030,9 +1030,9 @@ class Hub(SessionFactory):
             # build a fake header:
             header = {}
             header['engine'] = uuid
-            header['date'] = datetime.now()
+            header['date'] = util.utcnow()
             rec = dict(result_content=content, result_header=header, result_buffers=[])
-            rec['completed'] = header['date']
+            rec['completed'] = util.ensure_timezone(header['date'])
             rec['engine_uuid'] = uuid
             try:
                 self.db.update_record(msg_id, rec)
@@ -1097,9 +1097,8 @@ class Hub(SessionFactory):
         self.log.debug("save engine state to %s" % self.engine_state_file)
         state = {}
         engines = {}
-        for eid, ec in iteritems(self.engines):
-            if ec.uuid not in self.dead_engines:
-                engines[eid] = ec.uuid
+        for eid, ec in self.engines.items():
+            engines[eid] = ec.uuid
         
         state['engines'] = engines
         
@@ -1256,7 +1255,6 @@ class Hub(SessionFactory):
         """Resubmit one or more tasks."""
         parent = msg
         def finish(reply):
-            print("finishing resubmit")
             self.session.send(self.query, 'resubmit_reply', content=reply, ident=client_id, parent=parent)
 
         content = msg['content']
@@ -1458,7 +1456,7 @@ class Hub(SessionFactory):
                                             parent=msg, ident=client_id,
                                             buffers=buffers)
 
-    def become_distributed(self, client_id, msg):
+    def become_dask(self, client_id, msg):
         """Start a dask.distributed Scheduler."""
         if self.distributed_scheduler is None:
             kwargs = msg['content'].get('scheduler_args', {})
@@ -1474,7 +1472,7 @@ class Hub(SessionFactory):
             'port': self.distributed_scheduler.port,
         }
         self.log.info("dask.distributed scheduler running at {ip}:{port}".format(**content))
-        self.session.send(self.query, "become_distributed_reply", content=content,
+        self.session.send(self.query, "become_dask_reply", content=content,
             parent=msg, ident=client_id,
         )
 
